@@ -12,7 +12,6 @@
 
 #import "VLCOneDriveController.h"
 #import "VLCOneDriveConstants.h"
-#import "UIDevice+VLC.h"
 #import "NSString+SupportedMedia.h"
 #import <OneDriveSDK.h>
 
@@ -83,13 +82,30 @@ static void *ProgressObserverContext = &ProgressObserverContext;
 {
     _presentingViewController = presentingViewController;
 
+    if (@available(iOS 11.0, *)) {
+        [[UINavigationBar appearance] setPrefersLargeTitles:NO];
+    }
+
     [ODClient authenticatedClientWithCompletion:^(ODClient *client, NSError *error) {
+        if (@available(iOS 11.0, *)) {
+            [VLCAppearanceManager setupAppearanceWithTheme:PresentationTheme.current];
+        }
+        if (@available(iOS 13.0, *)) {
+            [VLCAppearanceManager setupUserInterfaceStyleWithTheme:PresentationTheme.current];
+        }
         if (error) {
             [self authFailed:error];
             return;
         }
         self->_oneDriveClient = client;
         [self authSuccess];
+
+        dispatch_async(dispatch_get_main_queue(), ^{
+            if (self.delegate) {
+                if ([self.delegate respondsToSelector:@selector(sessionWasUpdated)])
+                    [self.delegate performSelector:@selector(sessionWasUpdated)];
+            }
+        });
     }];
 }
 
@@ -105,6 +121,9 @@ static void *ProgressObserverContext = &ProgressObserverContext;
         self->_rootItemID = nil;
         self->_parentItem = nil;
         dispatch_async(dispatch_get_main_queue(), ^{
+            if (self.delegate) {
+                [self.delegate performSelector:@selector(mediaListUpdated)];
+            }
             if (self->_presentingViewController) {
                 [self->_presentingViewController.navigationController popViewControllerAnimated:YES];
             }
@@ -122,24 +141,33 @@ static void *ProgressObserverContext = &ProgressObserverContext;
     return _oneDriveClient != nil;
 }
 
+- (void)sessionWasUpdated
+{
+    if (self.delegate) {
+        if ([self.delegate respondsToSelector:@selector(sessionWasUpdated)])
+            [self.delegate performSelector:@selector(sessionWasUpdated)];
+    }
+    [[NSNotificationCenter defaultCenter] postNotificationName:VLCOneDriveControllerSessionUpdated object:self];
+}
+
 - (void)authSuccess
 {
     APLog(@"VLCOneDriveController: Authentication complete.");
 
     [self setupSession];
 
-    [[NSNotificationCenter defaultCenter] postNotificationName:VLCOneDriveControllerSessionUpdated object:self];
+    dispatch_async(dispatch_get_main_queue(), ^{
+        [self sessionWasUpdated];
+    });
 }
 
 - (void)authFailed:(NSError *)error
 {
     APLog(@"VLCOneDriveController: Authentication failure.");
 
-    if (self.delegate) {
-        if ([self.delegate respondsToSelector:@selector(sessionWasUpdated)])
-            [self.delegate performSelector:@selector(sessionWasUpdated)];
-    }
-    [[NSNotificationCenter defaultCenter] postNotificationName:VLCOneDriveControllerSessionUpdated object:self];
+    dispatch_async(dispatch_get_main_queue(), ^{
+        [self sessionWasUpdated];
+    });
 }
 
 #pragma mark - listing
@@ -168,10 +196,30 @@ static void *ProgressObserverContext = &ProgressObserverContext;
 
 }
 
+- (void)loadODItemsContinueWithRequest:(ODChildrenCollectionRequest *)request
+                                result:(NSMutableArray *)result
+{
+    __weak typeof(self) weakSelf = self;
+    [request getWithCompletion:^(ODCollection *response,
+                                 ODChildrenCollectionRequest *nextRequest, NSError *error) {
+        if (!error) {
+            [result addObjectsFromArray:response.value];
+            if (nextRequest != NULL) {
+                [weakSelf loadODItemsContinueWithRequest:nextRequest result:result];
+            } else {
+                [weakSelf sendMediaListUpdateWithContent:result completionHandler:NULL];
+            }
+        } else {
+            [weakSelf handleLoadODItemErrorWithError:error itemID:@"root"];
+        }
+    }];
+}
+
 - (void)loadODItemsWithCompletionHandler:(void (^)(void))completionHandler
 {
     NSString *itemID = _currentItem ? _currentItem.id : @"root";
     ODChildrenCollectionRequest * request = [[[[_oneDriveClient drive] items:itemID] children] request];
+    NSMutableArray<ODItem *> *requestContent = [[NSMutableArray alloc] init];
 
     // Clear all current
     [_currentItems removeAllObjects];
@@ -180,9 +228,11 @@ static void *ProgressObserverContext = &ProgressObserverContext;
 
     [request getWithCompletion:^(ODCollection *response, ODChildrenCollectionRequest *nextRequest, NSError *error) {
         if (!error) {
-            [self prepareODItems:response.value];
-            if (completionHandler) {
-                completionHandler();
+            if (nextRequest != NULL) {
+                [weakSelf loadODItemsContinueWithRequest:nextRequest result:requestContent];
+            } else {
+                [weakSelf sendMediaListUpdateWithContent:response.value
+                                       completionHandler:completionHandler];
             }
         } else {
             [weakSelf handleLoadODItemErrorWithError:error itemID:itemID];
@@ -190,6 +240,14 @@ static void *ProgressObserverContext = &ProgressObserverContext;
     }];
 }
 
+- (void)sendMediaListUpdateWithContent:(NSArray *)content completionHandler:(void (^)(void))completionHandler
+{
+    [self prepareODItems:content];
+    if (completionHandler) {
+        completionHandler();
+    }
+}
+
 - (void)handleLoadODItemErrorWithError:(NSError *)error itemID:(NSString *)itemID
 {
     __weak typeof(self) weakSelf = self;
