@@ -22,6 +22,7 @@
 #import "VLCRemoteControlService.h"
 #import "VLCMetadata.h"
 #import "VLCPlayerDisplayController.h"
+#import <stdatomic.h>
 
 #import "VLC-Swift.h"
 
@@ -57,7 +58,7 @@ NSString *const VLCPlaybackServicePlaybackPositionUpdated = @"VLCPlaybackService
     UIView *_actualVideoOutputView;
     UIView *_preBackgroundWrapperView;
 
-    BOOL _needsMetadataUpdate;
+    volatile atomic_bool _metadataUpdatePending;
     BOOL _mediaWasJustStarted;
     int _majorPositionChangeInProgress;
     BOOL _recheckForExistingThumbnail;
@@ -328,7 +329,11 @@ NSString *const VLCPlaybackServicePlaybackPositionUpdated = @"VLCPlaybackService
     if (_mediaPlayer.equalizerEnabled) {
         unsigned int profile = (unsigned int)[userDefaults integerForKey:kVLCSettingEqualizerProfile];
         [_mediaPlayer resetEqualizerFromProfile:profile];
+        /* make sure the preset's preamp setting is deployed */
         [_mediaPlayer setPreAmplification:[_mediaPlayer preAmplification]];
+    } else {
+        /* the default is 6 dB leading to audio compression */
+        [_mediaPlayer setPreAmplification:[userDefaults floatForKey:kVLCSettingDefaultPreampLevel]];
     }
 
     _mediaWasJustStarted = YES;
@@ -736,11 +741,7 @@ NSString *const VLCPlaybackServicePlaybackPositionUpdated = @"VLCPlaybackService
         } break;
         case VLCMediaPlayerStateEnded: {
             NSInteger nextIndex = [self nextMediaIndex];
-
-            if (nextIndex == -1) {
-                _sessionWillRestart = NO;
-                [self stopPlayback];
-            } else {
+            if (nextIndex > -1) {
                 [_listPlayer playItemAtNumber:@(nextIndex)];
                 [[NSNotificationCenter defaultCenter]
                  postNotificationName:VLCPlaybackServicePlaybackMetadataDidChange object:self];
@@ -1097,8 +1098,9 @@ NSString *const VLCPlaybackServicePlaybackPositionUpdated = @"VLCPlaybackService
 
 - (void)setAmplification:(CGFloat)amplification forBand:(unsigned int)index
 {
-    if (!_mediaPlayer.equalizerEnabled)
+    if (!_mediaPlayer.equalizerEnabled) {
         [_mediaPlayer setEqualizerEnabled:YES];
+    }
 
     [_mediaPlayer setAmplification:amplification forBand:index];
 
@@ -1122,6 +1124,7 @@ NSString *const VLCPlaybackServicePlaybackPositionUpdated = @"VLCPlaybackService
     if (profile == 0) {
         _mediaPlayer.equalizerEnabled = NO;
         [userDefaults setBool:YES forKey:kVLCSettingEqualizerProfileDisabled];
+        [_mediaPlayer setPreAmplification:[userDefaults floatForKey:kVLCSettingDefaultPreampLevel]];
         return;
     }
 
@@ -1242,9 +1245,10 @@ NSString *const VLCPlaybackServicePlaybackPositionUpdated = @"VLCPlaybackService
 
 - (void)setNeedsMetadataUpdate
 {
-    if (_needsMetadataUpdate == NO) {
-        _needsMetadataUpdate = YES;
+    if (!atomic_load(&_metadataUpdatePending)) {
+        atomic_store(&_metadataUpdatePending, true);
         dispatch_async(dispatch_get_global_queue(DISPATCH_QUEUE_PRIORITY_LOW, 0), ^{
+            atomic_store(&self->_metadataUpdatePending, false);
 #if TARGET_OS_IOS
             VLCMLMedia *media = self->_mediaPlayer.media ? [self->_delegate mediaForPlayingMedia:self->_mediaPlayer.media] : nil;
             [self->_metadata updateMetadataFromMedia:media mediaPlayer:self->_mediaPlayer];
@@ -1252,7 +1256,6 @@ NSString *const VLCPlaybackServicePlaybackPositionUpdated = @"VLCPlaybackService
             [self->_metadata updateMetadataFromMediaPlayer:self->_mediaPlayer];
 #endif
             dispatch_async(dispatch_get_main_queue(), ^{
-                self->_needsMetadataUpdate = NO;
                 [self recoverDisplayedMetadata];
             });
         });
